"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const set_1 = require("./set");
const freeze_1 = require("./freeze");
const internals_1 = require("../internals");
function fromPairs(pairs, comparator = internals_1.DEFAULT_COMPARATOR) {
    const tree = internals_1.createTree(true, comparator);
    var pair;
    if (Array.isArray(pairs)) {
        for (var i = 0; i < pairs.length; i++) {
            pair = pairs[i];
            set_1.set(pair[0], pair[1], tree);
        }
    }
    else {
        const it = pairs[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            pair = current.value;
            set_1.set(pair[0], pair[1], tree);
        }
    }
    return freeze_1.freeze(tree);
}
exports.fromPairs = fromPairs;
/**
 * Creates a new `RedBlackTree` from a plain input object. If no comparator function is supplied, keys are compared
 * using logical less-than and greater-than operations, which will generally only be suitable for numeric or string keys.
 *
 * @export
 * @template V The type of values in the tree
 * @param {Associative<V>} obj The input object from which to create a new tree
 * @param {Comparator<any>} [comparator] A comparison function, taking two keys, and returning a value less than 0 if the
 *                                     first key is smaller than the second, a value greater than 0 if the first key is
 *                                     greater than the second, or 0 if they're the same.
 * @returns {RedBlackTree<string, V>} A tree populated with the keys and values of the input object
 */
function fromObject(obj, comparator) {
    const tree = internals_1.createTree(true, comparator);
    const keys = Object.keys(obj);
    for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        set_1.set(key, obj[key], tree);
    }
    return freeze_1.freeze(tree);
}
exports.fromObject = fromObject;

//# sourceMappingURL=from.js.map
