"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const curry_1 = require("@typed/curry");
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
const index_1 = require("./index");
const NOMAP = () => { };
function arrayFrom(arg, tree) {
    var map;
    if (core_1.isUndefined(tree)) {
        tree = arg;
        map = NOMAP;
    }
    else {
        map = arg;
    }
    var array = new Array(index_1.size(tree));
    var it = index_1.iterateFromFirst(tree);
    var i, entry;
    if (map === NOMAP) {
        for (i = 0; i < array.length; i++) {
            array[i] = it.next().value;
        }
    }
    else {
        for (i = 0; i < array.length; i++) {
            var entry = it.next().value;
            array[i] = map(entry.value, entry.key, i);
        }
    }
    return array;
}
exports.arrayFrom = arrayFrom;
function values(tree) {
    var array = new Array(tree._size);
    var it = index_1.iterateFromFirst(tree);
    for (var i = 0; i < array.length; i++) {
        var node = it.next().value;
        array[i] = node.value;
    }
    return array;
}
exports.values = values;
function keys(tree) {
    var array = new Array(tree._size);
    var it = index_1.iterateFromFirst(tree);
    for (var i = 0; i < array.length; i++) {
        var node = it.next().value;
        array[i] = node.key;
    }
    return array;
}
exports.keys = keys;
const newObject = () => ({});
const unwrapShallow = curry_1.curry3(unwrapTree)(false);
const unwrapDeep = curry_1.curry3(core_1.preventCircularRefs)(newObject, curry_1.curry3(unwrapTree)(true));
/**
 * Returns the tree unwrapped as a plain JavaScript object. Keys are treated as (or converted to) strings.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {boolean} deep If true, any valid Collectable.js collection values nested in the tree will also be unwrapped
 * @param {RedBlackTree<K, V>} tree The tree to read values from
 * @returns {Associative<V>} A plain JavaScript object containing entries from the tree.
 */
function unwrap(deep, tree) {
    return deep ? unwrapDeep(tree) : unwrapShallow(tree, newObject());
}
exports.unwrap = unwrap;
function unwrapTree(deep, tree, target) {
    if (tree._size === 0) {
        return {};
    }
    var stack = new Array(Math.ceil(Math.log(tree._size)) * 2);
    var i = 0, node = tree._root, branch = 1 /* LEFT */;
    do {
        target[node.key] = deep ? core_1.unwrapAny(node.value) : node.value;
        if (branch === 1 /* LEFT */) {
            if (!internals_1.isNone(node._left)) {
                stack[i++] = [node, 2 /* RIGHT */];
                node = node._left;
                branch = 1 /* LEFT */;
                continue;
            }
            else {
                branch = 2 /* RIGHT */;
            }
        }
        if (branch === 2 /* RIGHT */) {
            if (!internals_1.isNone(node._right)) {
                stack[i++] = [node, 0 /* NONE */];
                node = node._right;
                branch = 1 /* LEFT */;
                continue;
            }
            else {
                branch = 0 /* NONE */;
            }
        }
        if (branch === 0 /* NONE */ && i > 0) {
            [node, branch] = stack[--i];
        }
    } while (i > 0 || branch !== 0 /* NONE */);
    return target;
}

//# sourceMappingURL=unwrap.js.map
